"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var MESSAGE_AUTOHIDE = 3 * 1000;
var ESC = 27;

var DEFAULT_MAP = {
  65: "LEFT",
  68: "RIGHT",
  87: "UP",
  83: "DOWN",
  80: "A",
  79: "B",
  81: "SELECT",
  69: "START"
};

var HUMAN_CODES = {
  37: 11104,
  38: 11105,
  39: 11106,
  40: 11107
};

var EVENT_TYPE_MAP = {
  keydown: true,
  keyup: false
};

function loadMap(key) {
  var map = localStorage.getItem(key);
  if (map) {
    try {
      return JSON.parse(map);
    } catch (e) {
      console.error("Map restore failed", e);
    }
  }
  return DEFAULT_MAP;
}

function saveMap(key, map) {
  window.localStorage.setItem(key, JSON.stringify(map));
}

function keyCodeToHuman(keyCode) {
  return HUMAN_CODES[keyCode] ? String.fromCharCode(HUMAN_CODES[keyCode]) : String.fromCharCode(keyCode) || keyCode;
}

var InputMapper = function (_Component) {
  _inherits(InputMapper, _Component);

  function InputMapper(props) {
    _classCallCheck(this, InputMapper);

    var _this = _possibleConstructorReturn(this, (InputMapper.__proto__ || Object.getPrototypeOf(InputMapper)).call(this, props));

    _this.state = {
      message: null,
      map: loadMap(props.localStorage),
      showMessage: false,
      showMap: false,
      waitingForInput: null
    };

    _this.keyOrder = [];

    _this.onInput = _this.onInput.bind(_this);
    return _this;
  }

  _createClass(InputMapper, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      window.addEventListener("keydown", this.onInput);
      window.addEventListener("keyup", this.onInput);

      this.svgNode.addEventListener("load", function () {
        _this2.initialize(_this2.svgNode.contentDocument);
      });
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      window.removeEventListener("keydown", this.onInput);
      window.removeEventListener("keyup", this.onInput);
    }
  }, {
    key: "initialize",
    value: function initialize(svgNode) {
      var _this3 = this;

      this.keyOrder = [];
      var nodes = svgNode.querySelectorAll("[id]");
      for (var i = 0, node; node = nodes[i]; ++i) {
        var id = node.getAttribute("id");
        if (id.startsWith("snex-button-")) {
          (function () {
            var name = id.replace("snex-button-", "");
            _this3.keyOrder.push(name);
            node.addEventListener("click", function () {
              return _this3.enableRemap(name);
            });
          })();
        }
      }
    }
  }, {
    key: "emitMessage",
    value: function emitMessage(message) {
      var _this4 = this;

      var hold = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      this.setState({
        message: message,
        showMessage: true
      });

      if (!hold) {
        clearTimeout(this.messageTimer);
        this.messageTimer = setTimeout(function () {
          _this4.cancelMessage();
        }, MESSAGE_AUTOHIDE);
      }
    }
  }, {
    key: "cancelMessage",
    value: function cancelMessage() {
      this.setState({
        showMessage: false
      });
    }
  }, {
    key: "enableRemap",
    value: function enableRemap(keyName) {
      var text = "Press any key to remap \"" + keyName + "\".";
      this.emitMessage(text, true);
      this.setState({
        showMap: false,
        waitingForInput: keyName
      });
      window.focus();
    }
  }, {
    key: "cancelRemap",
    value: function cancelRemap() {
      this.setState({
        waitingForInput: null
      });
    }
  }, {
    key: "handleInputWhileRemapping",
    value: function handleInputWhileRemapping(_ref) {
      var keyCode = _ref.keyCode;

      if (keyCode === ESC) {
        this.cancelMessage();
        this.cancelRemap();
        return;
      }

      var keyName = this.state.waitingForInput;
      var map = Object.assign({}, this.state.map, _defineProperty({}, keyCode, keyName));

      var text = "Mapped \"" + keyName + "\" to " + keyCodeToHuman(keyCode) + ".";
      this.cancelRemap();
      this.emitMessage(text, false);

      this.setState({ map: map });
      saveMap(this.props.localStorageKey, map);
    }
  }, {
    key: "onInput",
    value: function onInput(event) {
      if (this.state.waitingForInput) {
        return this.handleInputWhileRemapping(event);
      }

      var keyCode = event.keyCode;
      var map = this.state.map;

      if (map[keyCode]) {
        this.props.onInput({
          key: map[keyCode],
          state: EVENT_TYPE_MAP[event.type]
        });
      }
    }
  }, {
    key: "render",
    value: function render() {
      var _this5 = this;

      var map = this.state.map;

      var reverseMap = Object.keys(map).reduce(function (reverseMap, keyCode) {
        var key = map[keyCode];
        if (!reverseMap[key]) {
          reverseMap[key] = [];
        }
        reverseMap[map[keyCode]].push(keyCode);
        return reverseMap;
      }, {});

      return _react2.default.createElement(
        "div",
        { className: "snex-input-mapper", style: { position: "relative" } },
        _react2.default.createElement(
          "div",
          {
            className: "key-map",
            style: {
              top: "50%",
              left: "50%",
              transform: ["translateX(-50%)", "translateY(-50%)", "rotateX(" + (this.state.showMap ? 0 : 90) + "deg)"].join(" "),
              transition: "transform 0.5s ease",
              position: "absolute",
              zIndex: 2
            }
          },
          _react2.default.createElement(
            "table",
            null,
            _react2.default.createElement(
              "tbody",
              null,
              this.keyOrder.map(function (keyName) {
                return _react2.default.createElement(
                  "tr",
                  { key: keyName },
                  _react2.default.createElement(
                    "th",
                    null,
                    keyName
                  ),
                  _react2.default.createElement(
                    "td",
                    null,
                    reverseMap[keyName].map(function (keyCode) {
                      return keyCodeToHuman(keyCode);
                    }).join(" ")
                  )
                );
              })
            )
          ),
          _react2.default.createElement(
            "small",
            null,
            _react2.default.createElement(
              "a",
              {
                className: "hide-key-map",
                style: { cursor: "pointer" },
                onClick: function onClick() {
                  return _this5.setState({ showMap: false });
                }
              },
              "Hide Key Map"
            )
          )
        ),
        _react2.default.createElement(
          "div",
          { className: "controller", style: { position: "relative" } },
          _react2.default.createElement("object", {
            data: this.props.svgURL,
            type: "image/svg+xml",
            style: {
              width: "100%"
            },
            ref: function ref(node) {
              return _this5.svgNode = node;
            }
          })
        ),
        _react2.default.createElement(
          "div",
          { style: { display: "flex", position: "absolute", width: "100%" } },
          _react2.default.createElement(
            "div",
            {
              className: "message",
              style: {
                margin: "auto",
                textAlign: "center",
                top: "100%"
              }
            },
            this.state.showMessage ? this.state.message : _react2.default.createElement(
              "small",
              null,
              _react2.default.createElement(
                "a",
                {
                  className: "show-key-map",
                  style: {
                    cursor: "pointer",
                    opacity: this.state.showMap ? 0 : 1,
                    transition: "opacity 0.5s ease"
                  },
                  onClick: function onClick() {
                    return _this5.setState({ showMap: !_this5.state.showMap });
                  }
                },
                "Show Key Map"
              )
            )
          )
        )
      );
    }
  }]);

  return InputMapper;
}(_react.Component);

InputMapper.propTypes = {
  onInput: _propTypes2.default.func.isRequired,
  svgURL: _propTypes2.default.string.isRequired,
  localStorageKey: _propTypes2.default.string.isRequired
};
InputMapper.defaultProps = {
  localStorageKey: "snex-react-input-mapper"
};
exports.default = InputMapper;